import unittest
from datetime import date, datetime
from decimal import Decimal
from django_admin_search.utils import format_data
from django import forms
from django.db import models
from django.core.exceptions import ValidationError
from django_mock_queries.query import MockSet, MockModel


class TestUtilsFunctions(unittest.TestCase):

    def test_char_field_parse(self):
        field = forms.CharField()
        self.assertEqual(format_data(field, '1'), '1')
        self.assertEqual(format_data(field, 1), '1')
        self.assertEqual(format_data(field, False), 'False')
        self.assertEqual(format_data(field, 0), '0')
        self.assertEqual(format_data(field, '%20F'), '%20F')
        self.assertRaises(ValidationError, lambda: format_data(field, ' '))

    def test_text_field_parse(self):
        field = forms.TextInput()
        self.assertEqual(format_data(field, '1'), '1')
        self.assertEqual(format_data(field, 1), '1')
        self.assertEqual(format_data(field, False), 'False')
        self.assertEqual(format_data(field, 0), '0')
        self.assertEqual(format_data(field, '%20F'), '%20F')
        self.assertEqual(format_data(field, ' '), ' ')

    def test_choice_field_parse(self):
        field = forms.ChoiceField(choices=[(True, 'True'), (False, 'False')])
        self.assertEqual(format_data(field, True), True)
        self.assertEqual(format_data(field, False), False)
        self.assertRaises(ValidationError, lambda: format_data(field, 1))
        self.assertRaises(ValidationError, lambda: format_data(field, 0))
        self.assertRaises(ValidationError, lambda: format_data(field, 'true'))
        self.assertRaises(ValidationError, lambda: format_data(field, 'false'))

    def test_model_choice_field_parse(self):
        qs = MockSet(MockModel(pk=0), MockModel(pk=1), MockModel(pk=2))
        field = forms.ModelChoiceField(queryset=qs)
        
        self.assertEqual(format_data(field, 0), 0)
        self.assertEqual(format_data(field, 1), 1)
        self.assertEqual(format_data(field, 2), 2)
        self.assertRaises(AttributeError, lambda: format_data(field, 4))

    def test_boolean_field_parse(self):
        field = forms.BooleanField()
        self.assertEqual(format_data(field, True), True)
        self.assertEqual(format_data(field, 1), True)
        self.assertEqual(format_data(field, '1'), True)
        self.assertEqual(format_data(field, False), False)
        self.assertEqual(format_data(field, 0), False)
        self.assertEqual(format_data(field, ''), False)

    def test_float_field_parse(self):
        field = forms.FloatField()
        self.assertEqual(format_data(field, 1), 1.0)
        self.assertEqual(format_data(field, '1'), 1.0)
        self.assertEqual(format_data(field, 0), 0.0)
        self.assertEqual(format_data(field, '0'), 0.0)

    def test_decimal_field_parse(self):
        field = forms.DecimalField()

        self.assertEqual(format_data(field, 1), 1.0)
        self.assertEqual(isinstance(format_data(field, 1), Decimal), True)

        self.assertEqual(format_data(field, '1'), 1.0)
        self.assertEqual(isinstance(format_data(field, '1'), Decimal), True)

        self.assertEqual(format_data(field, 0), 0.0)
        self.assertEqual(isinstance(format_data(field, 0), Decimal), True)

        self.assertEqual(format_data(field, '0'), 0.0)
        self.assertEqual(isinstance(format_data(field, '0'), Decimal), True)

    def test_integer_field_parse(self):
        field = forms.IntegerField()

        self.assertEqual(format_data(field, 1), 1)
        self.assertEqual(format_data(field, '1'), 1)
        self.assertEqual(format_data(field, '0'), 0)
        self.assertRaises(ValidationError, lambda: format_data(field, 1.1))

    def test_date_field_parse(self):
        field = forms.DateField()
        self.assertEqual(format_data(field, '06/26/1997'), date(1997, 6, 26))
        self.assertRaises(ValidationError, lambda: format_data(field, ''))
        self.assertRaises(AttributeError, lambda: format_data(field, 1))

    def test_datetime_field_parse(self):
        field = forms.DateTimeField()
        self.assertEqual(format_data(field, '06/26/1997'), datetime(1997, 6, 26))
        self.assertEqual(format_data(field, '06/26/1997 3:0:0'), datetime(1997, 6, 26, 3, 0, 0))
        self.assertRaises(ValidationError, lambda: format_data(field, ''))
        self.assertRaises(AttributeError, lambda: format_data(field, 1))



import unittest
from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _
from django_admin_search.apps import DjangoAdminSearchConfig

class TestDjangoAppConfig(unittest.TestCase):

    def test_valid_subclass_appconfig(self):
        """
            This project use format major.minor.patch
        """
        self.assertEqual(issubclass(DjangoAdminSearchConfig, AppConfig), True)

    def test_valid_name(self):
        name = DjangoAdminSearchConfig.name
        self.assertEqual(isinstance(name, str), True)
        self.assertEqual(name, 'django_admin_search')

    def test_valid_verbose_name(self):
        verbose_name = DjangoAdminSearchConfig.verbose_name
        self.assertEqual(verbose_name, _('Django Admin Search'))


# -*- coding: utf-8 -*-
from django.apps import AppConfig
from django.utils.translation import ugettext_lazy as _


class DjangoAdminSearchConfig(AppConfig):  # Our app config class
    name = 'django_admin_search'
    verbose_name = _('Django Admin Search')



from django.contrib.admin.views.main import SEARCH_VAR
from django.template import Library
from django.template.loader import render_to_string

register = Library()  # pylint: disable=C0103


# @register.inclusion_tag('admin/custom_search_form.html', takes_context=True)
@register.simple_tag(takes_context=True)
def advanced_search_form(context, cl):
    """
    Displays a search form for searching the list.
    """
    if not context.get('asf', None):
        return ''

    context = {
        'asf': context['asf'],
        'cl': cl,
        'show_result_count': cl.result_count != cl.full_result_count,
        'search_var': SEARCH_VAR
    }

    return render_to_string('admin/custom_search_form.html', context)



import os
import django
from django.conf import settings

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

settings.configure()

settings.INSTALLED_APPS = [
    # Django apps
    'django.contrib.admin',
    'django.contrib.contenttypes',
    'django.contrib.staticfiles',

    # Third part apps
    'django_admin_search',
    'test'
]

settings.TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.join(BASE_DIR, 'templates'),
        ],
        'APP_DIRS': True,
    },
]

settings.STATIC_URL = '/'
django.setup()


from django import forms


def format_data(value, key_value):
    """
    Return data converted by form type
    """
    if isinstance(value, (forms.ChoiceField, forms.ModelChoiceField)):
        value.clean(key_value)
        return key_value

    elif isinstance(value, forms.TextInput):
        return str(key_value)

    elif isinstance(value, forms.BooleanField):
        # in case you have doubts (because i find it very strange)
        # see a ticket https://code.djangoproject.com/ticket/31049
        return bool(key_value)

    else:
        return value.clean(key_value)



# -*- coding: utf-8 -*-
#!/usr/bin/env python
from io import open

from setuptools import find_packages, setup

from django_admin_search.meta import VERSION

extras_require = {
    'dev': [
        'django',
        'django_mock_queries',
        'pylint',
        'pytest-pylint',
        'pytest',
        'pytest-cov',
        'pytest-watch'
    ]
}

setup(
    name='django-admin-search',
    version=str(VERSION),
    description='The "Django Admin Search" is a advanced search modal for django admin',
    long_description=open('README.md', encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    author='Shinneider Libanio da Silva',
    author_email='shinneider-libanio@hotmail.com',
    url='https://github.com/shinneider/django_admin_search',
    license='MIT',
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
    python_requires=">=3.3",
    extras_require=extras_require,
    install_requires=[
        'django',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]
)
